@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004, Jeremiah Fincher
+# Copyright (c) 2002-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -27,10 +27,6 @@
 # POSSIBILITY OF SUCH DAMAGE.
 ###
 
-__revision__ = "$Id: __init__.py,v 1.44 2005/01/08 07:22:46 jamessan Exp $"
-
-import supybot.fix as fix
-
 import gc
 import os
 import re
@@ -53,7 +49,6 @@ import supybot.utils as utils
 import supybot.world as world
 from supybot.commands import *
 import supybot.ircutils as ircutils
-import supybot.webutils as webutils
 import supybot.callbacks as callbacks
 
 try:
@@ -103,10 +98,11 @@ class NoSuitableDatabase(Exception):
         self.suitable.sort()
 
     def __str__(self):
-        return 'No suitable databases were found.  Suitable databases ' \
-               'include %s.  If you have one of these databases installed, ' \
-               'make sure it is listed in the supybot.databases ' \
-               'configuration variable.' % utils.commaAndify(self.suitable)
+        return format('No suitable databases were found.  Suitable databases '
+                      'include %L.  If you have one of these databases '
+                      'installed, make sure it is listed in the '
+                      'supybot.databases configuration variable.',
+                      self.suitable)
 
 def DB(filename, types):
     filename = conf.supybot.directories.data.dirize(filename)
@@ -122,41 +118,6 @@ def DB(filename, types):
         raise NoSuitableDatabase, types.keys()
     return MakeDB
 
-class DBHandler(object):
-    def __init__(self, name=None, suffix='.db'):
-        if name is None:
-            self.name = self.__class__.__name__
-        else:
-            self.name = name
-        if suffix and suffix[0] != '.':
-            suffix = '.' + suffix
-        self.suffix = suffix
-        self.cachedDb = None
-
-    def makeFilename(self):
-        if self.name.endswith(self.suffix):
-            return self.name
-        else:
-            return self.name + self.suffix
-
-    def makeDb(self, filename):
-        raise NotImplementedError
-
-    def getDb(self):
-        if self.cachedDb is None or \
-           threading.currentThread() is not world.mainThread:
-            db = self.makeDb(self.makeFilename())
-        else:
-            db = self.cachedDb
-        db.autocommit = 1
-        return db
-
-    def die(self):
-        if self.cachedDb is not None:
-            self.cachedDb.die()
-            del self.cachedDb
-
-
 def makeChannelFilename(filename, channel=None, dirname=None):
     assert channel is not None, 'Death to those who use None for their channel'
     filename = os.path.basename(filename)
@@ -228,7 +189,7 @@ class ChannelDBHandler(object):
 
 class DbiChannelDB(object):
     """This just handles some of the general stuff for Channel DBI databases.
-    Check out plugins/Dunno.py for an example of how to use this."""
+    Check out ChannelIdDatabasePlugin for an example of how to use this."""
     def __init__(self, filename):
         self.filename = filename
         self.dbs = ircutils.IrcDict()
@@ -257,8 +218,6 @@ class DbiChannelDB(object):
         return _getDbAndDispatcher
 
 
-# XXX This should eventually be gotten rid of in favor of some dbi thing.  At
-# the very least, it ought to get an interface much closer to dbi.DB.
 class ChannelUserDictionary(UserDict.DictMixin):
     IdDict = dict
     def __init__(self):
@@ -325,7 +284,7 @@ class ChannelUserDB(ChannelUserDictionary):
             log.debug('Exception: %s', utils.exnToString(e))
 
     def flush(self):
-        fd = utils.transactionalFile(self.filename, makeBackupIfSmaller=False)
+        fd = utils.file.AtomicFile(self.filename, makeBackupIfSmaller=False)
         writer = csv.writer(fd)
         items = self.items()
         if not items:
@@ -365,7 +324,7 @@ def getUserName(id):
     else:
         return id
 
-class ChannelIdDatabasePlugin(callbacks.Privmsg):
+class ChannelIdDatabasePlugin(callbacks.Plugin):
     class DB(DbiChannelDB):
         class DB(dbi.DB):
             class Record(dbi.Record):
@@ -378,9 +337,9 @@ class ChannelIdDatabasePlugin(callbacks.Privmsg):
                 record = self.Record(at=at, by=by, text=text, **kwargs)
                 return super(self.__class__, self).add(record)
 
-    def __init__(self):
+    def __init__(self, irc):
         self.__parent = super(ChannelIdDatabasePlugin, self)
-        self.__parent.__init__()
+        self.__parent.__init__(irc)
         self.db = DB(self.name(), {'flat': self.DB})()
 
     def die(self):
@@ -389,9 +348,9 @@ class ChannelIdDatabasePlugin(callbacks.Privmsg):
 
     def getCommandHelp(self, name):
         help = self.__parent.getCommandHelp(name)
-        help = help.replace('$Types', utils.pluralize(self.name()))
+        help = help.replace('$Types', format('%p', self.name()))
         help = help.replace('$Type', self.name())
-        help = help.replace('$types', utils.pluralize(self.name().lower()))
+        help = help.replace('$types', format('%p', self.name().lower()))
         help = help.replace('$type', self.name().lower())
         return help
 
@@ -442,8 +401,8 @@ class ChannelIdDatabasePlugin(callbacks.Privmsg):
     remove = wrap(remove, ['user', 'channeldb', 'id'])
 
     def searchSerializeRecord(self, record):
-        text = utils.quoted(utils.ellipsisify(record.text, 50))
-        return '#%s: %s' % (record.id, text)
+        text = utils.str.ellipsisify(record.text, 50)
+        return format('#%s: %q', record.id, text)
 
     def search(self, irc, msg, args, channel, optlist, glob):
         """[<channel>] [--{regexp,by} <value>] [<glob>]
@@ -471,10 +430,10 @@ class ChannelIdDatabasePlugin(callbacks.Privmsg):
             L.append(self.searchSerializeRecord(record))
         if L:
             L.sort()
-            irc.reply('%s found: %s' % (len(L), utils.commaAndify(L)))
+            irc.reply(format('%s found: %L', len(L), L))
         else:
-            irc.reply('No matching %s were found.' %
-                      utils.pluralize(self.name().lower()))
+            what = self.name().lower()
+            irc.reply(format('No matching %p were found.', what))
     search = wrap(search, ['channeldb',
                            getopts({'by': 'otherUser',
                                     'regexp': 'regexpMatcher'}),
@@ -482,10 +441,8 @@ class ChannelIdDatabasePlugin(callbacks.Privmsg):
 
     def showRecord(self, record):
         name = getUserName(record.by)
-        at = time.localtime(record.at)
-        timeS = time.strftime(conf.supybot.reply.format.time(), at)
-        return '%s #%s: %s (added by %s at %s)' % \
-               (self.name(), record.id, utils.quoted(record.text), name, timeS)
+        return format('%s #%s: %q (added by %s at %t)',
+                      self.name(), record.id, record.text, name, record.at)
 
     def get(self, irc, msg, args, channel, id):
         """[<channel>] <id>
@@ -526,8 +483,8 @@ class ChannelIdDatabasePlugin(callbacks.Privmsg):
         itself.
         """
         n = self.db.size(channel)
-        irc.reply('There %s %s in my database.' %
-                  (utils.be(n), utils.nItems(self.name().lower(), n)))
+        whats = self.name().lower()
+        irc.reply(format('There %b %n in my database.', n, (n, whats)))
     stats = wrap(stats, ['channeldb'])
 
 
@@ -584,15 +541,15 @@ class PeriodicFileDownloader(object):
         self.currentlyDownloading.add(filename)
         try:
             try:
-                infd = webutils.getUrlFd(url)
+                infd = utils.web.getUrlFd(url)
             except IOError, e:
                 self.log.warning('Error downloading %s: %s', url, e)
                 return
-            except webutils.WebError, e:
+            except utils.web.Error, e:
                 self.log.warning('Error downloading %s: %s', url, e)
                 return
             confDir = conf.supybot.directories.data()
-            newFilename = os.path.join(confDir, utils.mktemp())
+            newFilename = os.path.join(confDir, utils.file.mktemp())
             outfd = file(newFilename, 'wb')
             start = time.time()
             s = infd.read(4096)
@@ -639,4 +596,4 @@ class PeriodicFileDownloader(object):
 
 
 
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
