@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004, Jeremiah Fincher
+# Copyright (c) 2002-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -31,13 +31,10 @@
 Includes wrappers for commands.
 """
 
-__revision__ = "$Id: commands.py,v 1.66 2005/01/08 07:22:47 jamessan Exp $"
-
-import supybot.fix as fix
-
 import time
 import types
 import getopt
+import inspect
 import threading
 
 import supybot.log as log
@@ -47,9 +44,7 @@ import supybot.world as world
 import supybot.ircdb as ircdb
 import supybot.ircmsgs as ircmsgs
 import supybot.ircutils as ircutils
-import supybot.webutils as webutils
 import supybot.callbacks as callbacks
-import supybot.structures as structures
 
 
 ###
@@ -63,13 +58,13 @@ def thread(f):
     """Makes sure a command spawns a thread when called."""
     def newf(self, irc, msg, args, *L, **kwargs):
         if world.isMainThread():
-            t = callbacks.CommandThread(target=irc._callCommand,
-                                        args=(f.func_name, self),
-                                        kwargs=kwargs)
+            targetArgs = (self.callingCommand, irc, msg, args) + tuple(L)
+            t = callbacks.CommandThread(target=self._callCommand,
+                                        args=targetArgs, kwargs=kwargs)
             t.start()
         else:
             f(self, irc, msg, args, *L, **kwargs)
-    return utils.changeFunctionName(newf, f.func_name, f.__doc__)
+    return utils.python.changeFunctionName(newf, f.func_name, f.__doc__)
 
 class UrlSnarfThread(world.SupyThread):
     def __init__(self, *args, **kwargs):
@@ -82,8 +77,8 @@ class UrlSnarfThread(world.SupyThread):
     def run(self):
         try:
             super(UrlSnarfThread, self).run()
-        except webutils.WebError, e:
-            log.debug('Exception in urlSnarfer: %s' % utils.exnToString(e))
+        except utils.web.Error, e:
+            log.debug('Exception in urlSnarfer: %s', utils.exnToString(e))
 
 class SnarfQueue(ircutils.FloodQueue):
     timeout = conf.supybot.snarfThrottle
@@ -125,6 +120,9 @@ def urlSnarfer(f):
         def doSnarf():
             _snarfLock.acquire()
             try:
+                # This has to be *after* we've acquired the lock so we can be
+                # sure that all previous urlSnarfers have already run to
+                # completion.
                 if msg.repliedTo:
                     self.log.debug('Not snarfing, msg is already repliedTo.')
                     return
@@ -137,7 +135,7 @@ def urlSnarfer(f):
             L = list(L)
             t = UrlSnarfThread(target=doSnarf, url=url)
             t.start()
-    newf = utils.changeFunctionName(newf, f.func_name, f.__doc__)
+    newf = utils.python.changeFunctionName(newf, f.func_name, f.__doc__)
     return newf
 
 
@@ -171,16 +169,16 @@ def getInt(irc, msg, args, state, type='integer', p=None):
         i = _int(args[0])
         if p is not None:
             if not p(i):
-                irc.errorInvalid(type, args[0])
+                state.errorInvalid(type, args[0])
         state.args.append(i)
         del args[0]
     except ValueError:
-        irc.errorInvalid(type, args[0])
+        state.errorInvalid(type, args[0])
 
 def getNonInt(irc, msg, args, state, type='non-integer value'):
     try:
         i = _int(args[0])
-        irc.errorInvalid(type, args[0])
+        state.errorInvalid(type, args[0])
     except ValueError:
         state.args.append(args.pop(0))
 
@@ -193,7 +191,7 @@ def getFloat(irc, msg, args, state, type='floating point number'):
         state.args.append(float(args[0]))
         del args[0]
     except ValueError:
-        irc.errorInvalid(type, args[0])
+        state.errorInvalid(type, args[0])
 
 def getPositiveInt(irc, msg, args, state, *L):
     getInt(irc, msg, args, state,
@@ -229,14 +227,14 @@ def getExpiry(irc, msg, args, state):
         state.args.append(expires)
         del args[0]
     except ValueError:
-        irc.errorInvalid('number of seconds', args[0])
+        state.errorInvalid('number of seconds', args[0])
 
 def getBoolean(irc, msg, args, state):
     try:
-        state.args.append(utils.toBool(args[0]))
+        state.args.append(utils.str.toBool(args[0]))
         del args[0]
     except ValueError:
-        irc.errorInvalid('boolean', args[0])
+        state.errorInvalid('boolean', args[0])
 
 def getNetworkIrc(irc, msg, args, state, errorIfNoMatch=False):
     if args:
@@ -252,15 +250,15 @@ def getNetworkIrc(irc, msg, args, state, errorIfNoMatch=False):
 
 def getHaveOp(irc, msg, args, state, action='do that'):
     if state.channel not in irc.state.channels:
-        irc.error('I\'m not even in %s.' % state.channel, Raise=True)
+        state.error('I\'m not even in %s.' % state.channel, Raise=True)
     if not irc.state.channels[state.channel].isOp(irc.nick):
-        irc.error('I need to be opped to %s.' % action, Raise=True)
+        state.error('I need to be opped to %s.' % action, Raise=True)
 
 def validChannel(irc, msg, args, state):
     if irc.isChannel(args[0]):
         state.args.append(args.pop(0))
     else:
-        irc.errorInvalid('channel', args[0])
+        state.errorInvalid('channel', args[0])
 
 def getHostmask(irc, msg, args, state):
     if ircutils.isUserHostmask(args[0]):
@@ -271,7 +269,7 @@ def getHostmask(irc, msg, args, state):
             state.args.append(hostmask)
             del args[0]
         except KeyError:
-            irc.errorInvalid('nick or hostmask', args[0])
+            state.errorInvalid('nick or hostmask', args[0])
 
 def getBanmask(irc, msg, args, state):
     getHostmask(irc, msg, args, state)
@@ -282,11 +280,11 @@ def getUser(irc, msg, args, state):
     try:
         state.args.append(ircdb.users.getUser(msg.prefix))
     except KeyError:
-        irc.errorNotRegistered(Raise=True)
+        state.errorNotRegistered(Raise=True)
 
 def getOtherUser(irc, msg, args, state):
     if ircutils.isUserHostmask(args[0]):
-        irc.errorNoUser(args[0])
+        state.errorNoUser(args[0])
     try:
         state.args.append(ircdb.users.getUser(args[0]))
         del args[0]
@@ -297,7 +295,7 @@ def getOtherUser(irc, msg, args, state):
             state.args.append(ircdb.users.getUser(hostmask))
             del args[0]
         except (KeyError, callbacks.Error):
-            irc.errorNoUser(name=args[0])
+            state.errorNoUser(name=args[0])
 
 def _getRe(f):
     def get(irc, msg, args, state, convert=True):
@@ -318,24 +316,24 @@ def _getRe(f):
                 else:
                     state.args.append(s)
             else:
-                irc.errorInvalid('regular expression', s)
+                state.errorInvalid('regular expression', s)
         except IndexError:
             args[:] = original
-            irc.errorInvalid('regular expression', s)
+            state.errorInvalid('regular expression', s)
     return get
 
-getMatcher = _getRe(utils.perlReToPythonRe)
-getReplacer = _getRe(utils.perlReToReplacer)
+getMatcher = _getRe(utils.str.perlReToPythonRe)
+getReplacer = _getRe(utils.str.perlReToReplacer)
 
 def getNick(irc, msg, args, state):
     if ircutils.isNick(args[0]):
         if 'nicklen' in irc.state.supported:
             if len(args[0]) > irc.state.supported['nicklen']:
-                irc.errorInvalid('nick', args[0],
+                state.errorInvalid('nick', args[0],
                                  'That nick is too long for this server.')
         state.args.append(args.pop(0))
     else:
-        irc.errorInvalid('nick', args[0])
+        state.errorInvalid('nick', args[0])
 
 def getSeenNick(irc, msg, args, state, errmsg=None):
     try:
@@ -344,7 +342,7 @@ def getSeenNick(irc, msg, args, state, errmsg=None):
     except KeyError:
         if errmsg is None:
             errmsg = 'I haven\'t seen %s.' % args[0]
-        irc.error(errmsg, Raise=True)
+        state.error(errmsg, Raise=True)
 
 def getChannel(irc, msg, args, state):
     if args and irc.isChannel(args[0]):
@@ -370,7 +368,7 @@ def getChannelDb(irc, msg, args, state, **kwargs):
         channel = channelSpecific.link()
         if not conf.get(channelSpecific.link.allow, channel):
             log.warning('channelSpecific.link is globally set to %s, but '
-                        '%s disallows linking to its db.' % (channel, channel))
+                        '%s disallowed linking to its db.', channel, channel)
             raise
         else:
             channel = channelSpecific.getChannelLink(channel)
@@ -381,12 +379,12 @@ def inChannel(irc, msg, args, state):
     if not state.channel:
         getChannel(irc, msg, args, state)
     if state.channel not in irc.state.channels:
-        irc.error('I\'m not in %s.' % state.channel, Raise=True)
+        state.error('I\'m not in %s.' % state.channel, Raise=True)
 
 def onlyInChannel(irc, msg, args, state):
     if not (irc.isChannel(msg.args[0]) and msg.args[0] in irc.state.channels):
-        irc.error('This command may only be given in a channel that I am in.',
-                  Raise=True)
+        state.error('This command may only be given in a channel that I am in.',
+                    Raise=True)
     else:
         state.channel = msg.args[0]
         state.args.append(state.channel)
@@ -398,16 +396,18 @@ def callerInGivenChannel(irc, msg, args, state):
             if msg.nick in irc.state.channels[channel].users:
                 state.args.append(args.pop(0))
             else:
-                irc.error('You must be in %s.' % channel, Raise=True)
+                state.error('You must be in %s.' % channel, Raise=True)
         else:
-            irc.error('I\'m not in %s.' % channel, Raise=True)
+            state.error('I\'m not in %s.' % channel, Raise=True)
     else:
-        irc.errorInvalid('channel', args[0])
+        state.errorInvalid('channel', args[0])
 
 def nickInChannel(irc, msg, args, state):
+    originalArgs = state.args[:]
     inChannel(irc, msg, args, state)
+    state.args = originalArgs
     if args[0] not in irc.state.channels[state.channel].users:
-        irc.error('%s is not in %s.' % (args[0], state.channel), Raise=True)
+        state.error('%s is not in %s.' % (args[0], state.channel), Raise=True)
     state.args.append(args.pop(0))
 
 def getChannelOrNone(irc, msg, args, state):
@@ -422,7 +422,16 @@ def checkChannelCapability(irc, msg, args, state, cap):
     cap = ircdb.canonicalCapability(cap)
     cap = ircdb.makeChannelCapability(state.channel, cap)
     if not ircdb.checkCapability(msg.prefix, cap):
-        irc.errorNoCapability(cap, Raise=True)
+        state.errorNoCapability(cap, Raise=True)
+
+def getOp(irc, msg, args, state):
+    checkChannelCapability(irc, msg, args, state, 'op')
+
+def getHalfop(irc, msg, args, state):
+    checkChannelCapability(irc, msg, args, state, 'halfop')
+
+def getVoice(irc, msg, args, state):
+    checkChannelCapability(irc, msg, args, state, 'voice')
 
 def getLowered(irc, msg, args, state):
     state.args.append(ircutils.toLower(args.pop(0)))
@@ -433,7 +442,7 @@ def getSomething(irc, msg, args, state, errorMsg=None, p=None):
     if not args[0] or not p(args[0]):
         if errorMsg is None:
             errorMsg = 'You must not give the empty string as an argument.'
-        irc.error(errorMsg, Raise=True)
+        state.error(errorMsg, Raise=True)
     else:
         state.args.append(args.pop(0))
 
@@ -444,18 +453,18 @@ def getSomethingNoSpaces(irc, msg, args, state, *L):
 
 def private(irc, msg, args, state):
     if irc.isChannel(msg.args[0]):
-        irc.errorRequiresPrivacy(Raise=True)
+        state.errorRequiresPrivacy(Raise=True)
 
 def public(irc, msg, args, state, errmsg=None):
     if not irc.isChannel(msg.args[0]):
         if errmsg is None:
             errmsg = 'This message must be sent in a channel.'
-        irc.error(errmsg, Raise=True)
+        state.error(errmsg, Raise=True)
 
 def checkCapability(irc, msg, args, state, cap):
     cap = ircdb.canonicalCapability(cap)
     if not ircdb.checkCapability(msg.prefix, cap):
-        irc.errorNoCapability(cap, Raise=True)
+        state.errorNoCapability(cap, Raise=True)
 
 def owner(irc, msg, args, state):
     checkCapability(irc, msg, args, state, 'owner')
@@ -473,37 +482,43 @@ def getGlob(irc, msg, args, state):
     state.args.append(glob)
 
 def getUrl(irc, msg, args, state):
-    if webutils.urlRe.match(args[0]):
+    if utils.web.urlRe.match(args[0]):
         state.args.append(args.pop(0))
     else:
-        irc.errorInvalid('url', args[0])
+        state.errorInvalid('url', args[0])
+
+def getEmail(irc, msg, args, state):
+    if utils.net.emailRe.match(args[0]):
+        state.args.append(args.pop(0))
+    else:
+        state.errorInvalid('email', args[0])
 
 def getHttpUrl(irc, msg, args, state):
-    if webutils.urlRe.match(args[0]) and args[0].startswith('http://'):
+    if utils.web.httpUrlRe.match(args[0]):
         state.args.append(args.pop(0))
     else:
-        irc.errorInvalid('http url', args[0])
+        state.errorInvalid('http url', args[0])
 
 def getNow(irc, msg, args, state):
     state.args.append(int(time.time()))
 
 def getCommandName(irc, msg, args, state):
     if ' ' in args[0]:
-        irc.errorInvalid('command name', args[0])
+        state.errorInvalid('command name', args[0])
     else:
         state.args.append(callbacks.canonicalName(args.pop(0)))
 
 def getIp(irc, msg, args, state):
-    if utils.isIP(args[0]):
+    if utils.net.isIP(args[0]):
         state.args.append(args.pop(0))
     else:
-        irc.errorInvalid('ip', args[0])
+        state.errorInvalid('ip', args[0])
 
 def getLetter(irc, msg, args, state):
     if len(args[0]) == 1:
         state.args.append(args.pop(0))
     else:
-        irc.errorInvalid('letter', args[0])
+        state.errorInvalid('letter', args[0])
 
 def getMatch(irc, msg, args, state, regexp, errmsg):
     m = regexp.search(args[0])
@@ -511,7 +526,7 @@ def getMatch(irc, msg, args, state, regexp, errmsg):
         state.args.append(m)
         del args[0]
     else:
-        irc.error(errmsg, Raise=True)
+        state.error(errmsg, Raise=True)
 
 def getLiteral(irc, msg, args, state, literals, errmsg=None):
     # ??? Should we allow abbreviations?
@@ -521,7 +536,7 @@ def getLiteral(irc, msg, args, state, literals, errmsg=None):
     if args[0] in abbrevs:
         state.args.append(abbrevs[args.pop(0)])
     elif errmsg is not None:
-        irc.error(errmsg, Raise=True)
+        state.error(errmsg, Raise=True)
     else:
         raise callbacks.ArgumentError
 
@@ -535,7 +550,7 @@ def getPlugin(irc, msg, args, state, require=True):
         state.args.append(cb)
         del args[0]
     elif require:
-        irc.errorInvalid('plugin', args[0])
+        state.errorInvalid('plugin', args[0])
     else:
         state.args.append(None)
 
@@ -543,7 +558,7 @@ def getIrcColor(irc, msg, args, state):
     if args[0] in ircutils.mircColors:
         state.args.append(ircutils.mircColors[args.pop(0)])
     else:
-        irc.errorInvalid('irc color')
+        state.errorInvalid('irc color')
 
 def getText(irc, msg, args, state):
     if args:
@@ -560,6 +575,7 @@ wrappers = ircutils.IrcDict({
     'color': getIrcColor,
     'now': getNow,
     'url': getUrl,
+    'email': getEmail,
     'httpUrl': getHttpUrl,
     'long': getLong,
     'float': getFloat,
@@ -605,6 +621,9 @@ wrappers = ircutils.IrcDict({
     'admin': admin,
     'checkCapability': checkCapability,
     'checkChannelCapability': checkChannelCapability,
+    'op': getOp,
+    'halfop': getHalfop,
+    'voice': getVoice,
 })
 
 def addConverter(name, wrapper):
@@ -673,7 +692,7 @@ class rest(context):
             except Exception, e:
                 args[:] = original
         else:
-            raise callbacks.ArgumentError
+            raise IndexError
 
 # additional means:  Look for this (and make sure it's of this type).  If
 # there are no arguments for us to check, then use our default.
@@ -698,6 +717,7 @@ class optional(additional):
             super(optional, self).__call__(irc, msg, args, state)
         except (callbacks.ArgumentError, callbacks.Error), e:
             log.debug('Got %s, returning default.', utils.exnToString(e))
+            state.errored = False
             setDefault(state, self.default)
 
 class any(context):
@@ -733,18 +753,23 @@ class first(context):
         if 'default' in kw:
             self.default = kw.pop('default')
             assert not kw, 'Bad kwargs for first.__init__'
+        self.spec = specs # for __repr__
         self.specs = map(contextify, specs)
 
     def __call__(self, irc, msg, args, state):
+        errored = False
         for spec in self.specs:
             try:
                 spec(irc, msg, args, state)
                 return
             except Exception, e:
+                errored = state.errored
+                state.errored = False
                 continue
         if hasattr(self, 'default'):
             state.args.append(self.default)
         else:
+            state.errored = errored
             raise e
 
 class reverse(context):
@@ -818,6 +843,14 @@ class State(object):
         self.kwargs = {}
         self.types = types
         self.channel = None
+        self.errored = False
+
+    def __getattr__(self, attr):
+        if attr.startswith('error'):
+            self.errored = True
+            return getattr(dynamic.irc, attr)
+        else:
+            raise AttributeError, attr
 
     def essence(self):
         st = State(self.types)
@@ -845,7 +878,7 @@ class Spec(object):
     def __init__(self, types, allowExtra=False):
         self.types = types
         self.allowExtra = allowExtra
-        utils.mapinto(contextify, self.types)
+        utils.seq.mapinto(contextify, self.types)
 
     def __call__(self, irc, msg, args, stateAttrs={}):
         state = self._state(self.types[:], stateAttrs)
@@ -860,14 +893,25 @@ class Spec(object):
             raise callbacks.ArgumentError
         return state
 
-def wrap(f, specList=[], **kw):
+def wrap(f, specList=[], name=None, **kw):
+    name = name or f.func_name
     spec = Spec(specList, **kw)
     def newf(self, irc, msg, args, **kwargs):
         state = spec(irc, msg, args, stateAttrs={'cb': self, 'log': self.log})
-        self.log.debug('State before call: %s' % state)
-        f(self, irc, msg, args, *state.args, **state.kwargs)
-    return utils.changeFunctionName(newf, f.func_name, f.__doc__)
-
+        self.log.debug('State before call: %s', state)
+        if state.errored:
+            self.log.debug('Refusing to call %s due to state.errored.', f)
+        else:
+            try:
+                f(self, irc, msg, args, *state.args, **state.kwargs)
+            except TypeError:
+                self.log.error('Spec: %s', specList)
+                self.log.error('Received args: %s', args)
+                code = f.func_code
+                funcArgs = inspect.getargs(code)[0][len(self.commandArgs):]
+                self.log.error('Extra args: %s', funcArgs)
+                raise
+    return utils.python.changeFunctionName(newf, name, f.__doc__)
 
 __all__ = [
     # Contexts.
@@ -890,4 +934,4 @@ __all__ = [
 ## if world.testing:
 ##     __all__.append('Spec')
 
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
