@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004, Jeremiah Fincher
+# Copyright (c) 2002-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -27,15 +27,10 @@
 # POSSIBILITY OF SUCH DAMAGE.
 ###
 
-__revision__ = "$Id: conf.py,v 1.239 2005/01/18 19:46:13 jamessan Exp $"
-
-import supybot.fix as fix
-
 import os
 import sys
 import time
 import socket
-import string
 
 import supybot.cdb as cdb
 import supybot.utils as utils
@@ -43,13 +38,12 @@ import supybot.registry as registry
 import supybot.ircutils as ircutils
 
 installDir = os.path.dirname(sys.modules[__name__].__file__)
-_srcDir = installDir
 _pluginsDir = os.path.join(installDir, 'plugins')
 
 ###
 # version: This should be pretty obvious.
 ###
-version = '0.80.1'
+version = '0.83.1'
 
 ###
 # *** The following variables are affected by command-line options.  They are
@@ -72,6 +66,12 @@ daemonized = False
 ###
 allowDefaultOwner = False
 
+###
+# Here we replace values in other modules as appropriate.
+###
+utils.web.defaultHeaders['User-agent'] = \
+                         'Mozilla/5.0 (Compatible; Supybot %s)' % version
+
 ###
 # The standard registry.
 ###
@@ -161,6 +161,7 @@ class ValidNicksAllowingPercentS(ValidNicks):
 class ValidChannel(registry.String):
     """Value must be a valid IRC channel name."""
     def setValue(self, v):
+        self.channel = v
         if ',' in v:
             # To prevent stupid users from: a) trying to add a channel key
             # with a comma in it, b) trying to add channels separated by
@@ -176,6 +177,20 @@ class ValidChannel(registry.String):
         else:
             registry.String.setValue(self, v)
 
+    def error(self):
+        try:
+            super(ValidChannel, self).error()
+        except registry.InvalidRegistryValue, e:
+            e.channel = self.channel
+            raise e
+
+class ValidHostmask(registry.String):
+    """Value must be a valid user hostmask."""
+    def setValue(self, v):
+        if not ircutils.isUserHostmask(v):
+            self.error()
+        super(ValidHostmask, self).setValue(v)
+
 registerGlobalValue(supybot, 'nick',
    ValidNick('supybot', """Determines the bot's default nick."""))
 
@@ -256,6 +271,9 @@ def registerNetwork(name, password=''):
         completed.""" % name))
     registerGlobalValue(network, 'channels', SpaceSeparatedSetOfChannels([],
         """Determines what channels the bot will join only on %s.""" % name))
+    registerGlobalValue(network, 'ssl', registry.Boolean(False,
+        """Determines whether the bot will attempt to connect with SSL sockets
+        to %s.""" % name))
     registerChannelValue(network.channels, 'key', registry.String('',
         """Determines what key (if any) will be used to join the channel."""))
     return network
@@ -280,6 +298,13 @@ registerChannelValue(supybot.reply.format, 'time',
     for human reading should be formatted. Refer to the Python documentation
     for the time module to see valid formatting characters for time
     formats."""))
+def timestamp(t):
+    if t is None:
+        t = time.time()
+    t = time.localtime(t)
+    format = get(supybot.reply.format.time, dynamic.channel)
+    return time.strftime(format, t)
+utils.str.timestamp = timestamp
 
 registerGroup(supybot.reply.format.time, 'elapsed')
 registerChannelValue(supybot.reply.format.time.elapsed, 'short',
@@ -338,7 +363,7 @@ registerChannelValue(supybot.reply.error, 'inPrivate',
     registry.Boolean(False, """Determines whether the bot will send error
     messages to users in private.  You might want to do this in order to keep
     channel traffic to minimum.  This can be used in combination with
-    supybot.reply.errorWithNotice."""))
+    supybot.reply.error.withNotice."""))
 registerChannelValue(supybot.reply.error, 'withNotice',
     registry.Boolean(False, """Determines whether the bot will send error
     messages to users via NOTICE instead of PRIVMSG.  You might want to do this
@@ -407,7 +432,7 @@ registerChannelValue(supybot.reply, 'showSimpleSyntax',
 class ValidPrefixChars(registry.String):
     """Value must contain only ~!@#$%^&*()_-+=[{}]\\|'\";:,<.>/?"""
     def setValue(self, v):
-        if v.translate(string.ascii, '`~!@#$%^&*()_-+=[{}]\\|\'";:,<.>/?'):
+        if v.translate(utils.str.chars, '`~!@#$%^&*()_-+=[{}]\\|\'";:,<.>/?'):
             self.error()
         registry.String.setValue(self, v)
 
@@ -474,9 +499,9 @@ registerChannelValue(supybot.replies, 'notRegistered',
     registry.NormalizedString("""You must be registered to use this command.
     If you are already registered, you must either identify (using the identify
     command) or add a hostmask matching your current hostmask (using the
-    addhostmask command).""", """Determines what error message the bot replies
-    with when someone tries to do something that requires them to be registered
-    but they're not currently recognized."""))
+    "hostmask add" command).""", """Determines what error message the bot
+    replies with when someone tries to do something that requires them to be
+    registered but they're not currently recognized."""))
 
 registerChannelValue(supybot.replies, 'noCapability',
     registry.NormalizedString("""You don't have the %s capability.  If you
@@ -581,12 +606,12 @@ registerChannelValue(supybot.commands.nested, 'pipeSyntax',
     example: 'bot: foo | bar'."""))
 
 registerGroup(supybot.commands, 'defaultPlugins',
-    orderAlphabetically=True, help=utils.normalizeWhitespace("""Determines
-    what commands have default plugins set, and which plugins are set to
-    be the default for each of those commands."""))
+    orderAlphabetically=True, help="""Determines what commands have default
+    plugins set, and which plugins are set to be the default for each of those
+    commands.""")
 registerGlobalValue(supybot.commands.defaultPlugins, 'importantPlugins',
     registry.SpaceSeparatedSetOfStrings(
-        ['Admin', 'Channel', 'Config', 'Misc', 'Owner', 'User'],
+        ['Admin', 'Channel', 'Config', 'Misc', 'Owner', 'Plugin', 'User'],
         """Determines what plugins automatically get precedence over all other
         plugins when selecting a default plugin for a command.  By default,
         this includes the standard loaded plugins.  You probably shouldn't
@@ -640,8 +665,7 @@ registerGlobalValue(supybot.drivers, 'poll',
     driver should block waiting for input."""))
 
 class ValidDriverModule(registry.OnlySomeStrings):
-    validStrings = ('default', 'socketDrivers',
-                    'twistedDrivers', 'asyncoreDrivers')
+    validStrings = ('default', 'Socket', 'Twisted')
 
 registerGlobalValue(supybot.drivers, 'module',
     ValidDriverModule('default', """Determines what driver module the bot will
@@ -651,6 +675,11 @@ registerGlobalValue(supybot.drivers, 'module',
     applications.  twistedDrivers is very stable and simple, and if you've got
     Twisted installed, is probably your best bet."""))
 
+registerGlobalValue(supybot.drivers, 'maxReconnectWait',
+    registry.PositiveFloat(300.0, """Determines the maximum time the bot will
+    wait before attempting to reconnect to an IRC server.  The bot may, of
+    course, reconnect earlier if possible."""))
+
 ###
 # supybot.directories, for stuff relating to directories.
 ###
@@ -707,18 +736,12 @@ registerGlobalValue(supybot.directories.data, 'tmp',
     DataFilenameDirectory('tmp', """Determines what directory temporary files
     are put into."""))
 
-# Remember, we're *meant* to replace this nice little wrapper.
-def transactionalFile(*args, **kwargs):
-    kwargs['tmpDir'] = supybot.directories.data.tmp()
-    kwargs['backupDir'] = supybot.directories.backup()
-    return utils.AtomicFile(*args, **kwargs)
-utils.transactionalFile = transactionalFile
+utils.file.AtomicFile.default.tmpDir = supybot.directories.data.tmp
+utils.file.AtomicFile.default.backupDir = supybot.directories.backup
 
 class PluginDirectories(registry.CommaSeparatedListOfStrings):
     def __call__(self):
         v = registry.CommaSeparatedListOfStrings.__call__(self)
-        if _srcDir not in v:
-            v.append(_srcDir)
         if _pluginsDir not in v:
             v.append(_pluginsDir)
         return v
@@ -916,12 +939,18 @@ registerGlobalValue(supybot.protocols.irc.ping, 'interval',
     registry.Integer(120, """Determines the number of seconds between sending
     pings to the server, if pings are being sent to the server."""))
 
-registerGlobalValue(supybot.protocols.irc, 'queueDuplicateMessages',
-    registry.Boolean(False, """Determines whether the bot will allow duplicate
+registerGroup(supybot.protocols.irc, 'queuing')
+registerGlobalValue(supybot.protocols.irc.queuing, 'duplicates',
+    registry.Boolean(False, """Determines whether the bot will refuse duplicate
     messages to be queued for delivery to the server.  This is a safety
     mechanism put in place to prevent plugins from sending the same message
-    multiple times; most of the time it doesn't matter, but when it does,
-    you'll probably want it to disallowed."""))
+    multiple times; most of the time it doesn't matter, unless you're doing
+    certain kinds of plugin hacking."""))
+
+registerGroup(supybot.protocols.irc.queuing, 'rateLimit')
+registerGlobalValue(supybot.protocols.irc.queuing.rateLimit, 'join',
+    registry.Float(0, """Determines how many seconds must elapse between JOINs
+    sent to the server."""))
 
 ###
 # supybot.protocols.http
@@ -936,6 +965,7 @@ registerGlobalValue(supybot.protocols.http, 'peekSize',
 registerGlobalValue(supybot.protocols.http, 'proxy',
     registry.String('', """Determines what proxy all HTTP requests should go
     through.  The value should be of the form 'host:port'."""))
+utils.web.proxy = supybot.protocols.http.proxy
 
 
 ###
@@ -950,7 +980,7 @@ registerGlobalValue(supybot, 'defaultIgnore',
 class IP(registry.String):
     """Value must be a valid IP."""
     def setValue(self, v):
-        if v and not (utils.isIP(v) or utils.isIPV6(v)):
+        if v and not (utils.net.isIP(v) or utils.net.isIPV6(v)):
             self.error()
         else:
             registry.String.setValue(self, v)
@@ -992,9 +1022,6 @@ registerGlobalValue(supybot.debug, 'flushVeryOften',
     registry.Boolean(False, """Determines whether the bot will automatically
     flush all flushers *very* often.  Useful for debugging when you don't know
     what's breaking or when, but think that it might be logged."""))
-registerGlobalValue(supybot.debug, 'generated',
-    registry.String('$Id: conf.py,v 1.239 2005/01/18 19:46:13 jamessan Exp $; %s' % time.ctime(), """Determines when this
-    configuration file was generated; it should be modified by
-    supybot-wizard"""))
 
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
