@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004, Jeremiah Fincher
+# Copyright (c) 2002-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -31,18 +31,12 @@
 Module for some slight database-independence for simple databases.
 """
 
-__revision__ = "$Id: dbi.py,v 1.19 2005/01/14 14:46:52 jemfinch Exp $"
-
-import supybot.fix as fix
-
 import csv
 import math
-import sets
-import random
-from itertools import ilen
 
 import supybot.cdb as cdb
 import supybot.utils as utils
+from supybot.utils.iter import ilen
 
 class Error(Exception):
     """General error for this module."""
@@ -262,8 +256,7 @@ class FlatfileMapping(MappingInterface):
 
     def vacuum(self):
         infd = file(self.filename)
-        outfd = utils.transactionalFile(self.filename,
-                                        makeBackupIfSmaller=False)
+        outfd = utils.file.AtomicFile(self.filename,makeBackupIfSmaller=False)
         outfd.write(infd.readline()) # First line, nextId.
         for line in infd:
             if not line.startswith('-'):
@@ -286,7 +279,7 @@ class CdbMapping(MappingInterface):
             self.db['nextId'] = '1'
 
     def _openCdb(self, *args, **kwargs):
-        self.db = cdb.open(filename, 'c', **kwargs)
+        self.db = cdb.open(self.filename, 'c', **kwargs)
 
     def _getNextId(self):
         i = int(self.db['nextId'])
@@ -369,7 +362,7 @@ class DB(object):
 
     def random(self):
         try:
-            return self._newRecord(*random.choice(self.map))
+            return self._newRecord(*utils.iter.choice(self.map))
         except IndexError:
             return None
 
@@ -413,7 +406,7 @@ class Record(object):
                 default = None
             self.defaults[name] = default
             self.converters[name] = converter
-        seen = sets.Set()
+        seen = set()
         for (name, value) in kwargs.iteritems():
             assert name in self.fields, 'name must be a record value.'
             seen.add(name)
@@ -429,7 +422,7 @@ class Record(object):
         return csv.join([repr(getattr(self, name)) for name in self.fields])
 
     def deserialize(self, s):
-        unseenRecords = sets.Set(self.fields)
+        unseenRecords = set(self.fields)
         for (name, strValue) in zip(self.fields, csv.split(s)):
             setattr(self, name, self.converters[name](strValue))
             unseenRecords.remove(name)
@@ -437,4 +430,4 @@ class Record(object):
             setattr(self, name, self.defaults[name])
 
     
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
