@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004, Jeremiah Fincher
+# Copyright (c) 2002-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -27,18 +27,11 @@
 # POSSIBILITY OF SUCH DAMAGE.
 ###
 
-__revision__ = "$Id: ircdb.py,v 1.103 2005/01/06 15:55:09 jamessan Exp $"
-
 from __future__ import division
 
-import supybot.fix as fix
-
 import os
-import sets
 import time
-import string
 import operator
-from itertools import imap, ilen, ifilter
 
 import supybot.log as log
 import supybot.conf as conf
@@ -47,6 +40,7 @@ import supybot.world as world
 import supybot.ircutils as ircutils
 import supybot.registry as registry
 import supybot.unpreserve as unpreserve
+from utils.iter import imap, ilen, ifilter
 
 def isCapability(capability):
     return len(capability.split(None, 1)) == 1
@@ -114,10 +108,10 @@ def canonicalCapability(capability):
     return capability.lower()
 
 def unWildcardHostmask(hostmask):
-    return hostmask.translate(string.ascii, '!@*?')
+    return hostmask.translate(utils.str.chars, '!@*?')
 
 _invert = invertCapability
-class CapabilitySet(sets.Set):
+class CapabilitySet(set):
     """A subclass of set handling basic capability stuff."""
     def __init__(self, capabilities=()):
         self.__parent = super(CapabilitySet, self)
@@ -225,11 +219,10 @@ class IrcUser(object):
             self.hostmasks = hostmasks
 
     def __repr__(self):
-        return '%s(id=%s, ignore=%s, password="", name=%s, hashed=%r, ' \
-               'capabilities=%r, hostmasks=[], secure=%r)\n' % \
-               (self.__class__.__name__, self.id, self.ignore,
-                utils.quoted(self.name), self.hashed, self.capabilities,
-                self.secure)
+        return format('%s(id=%s, ignore=%s, password="", name=%q, hashed=%r, '
+                      'capabilities=%r, hostmasks=[], secure=%r)\n',
+                      self.__class__.__name__, self.id, self.ignore,
+                      self.name, self.hashed, self.capabilities, self.secure)
 
     def __hash__(self):
         return hash(self.id)
@@ -546,7 +539,7 @@ class IrcChannelCreator(Creator):
 
     def ban(self, rest, lineno):
         self._checkId()
-        (pattern, expiration) = rest
+        (pattern, expiration) = rest.split()
         self.c.bans[pattern] = int(float(expiration))
 
     def ignore(self, rest, lineno):
@@ -611,7 +604,7 @@ class UsersDictionary(utils.IterableMap):
             if self.filename is not None:
                 L = self.users.items()
                 L.sort()
-                fd = utils.transactionalFile(self.filename)
+                fd = utils.file.AtomicFile(self.filename)
                 for (id, u) in L:
                     fd.write('user %s' % id)
                     fd.write(os.linesep)
@@ -620,7 +613,7 @@ class UsersDictionary(utils.IterableMap):
             else:
                 log.error('UsersDictionary.flush called with no filename.')
         else:
-            log.debug('Not flushing UsersDictionary becuase of noFlush.')
+            log.debug('Not flushing UsersDictionary because of noFlush.')
 
     def close(self):
         self.flush()
@@ -648,7 +641,7 @@ class UsersDictionary(utils.IterableMap):
                     try:
                         self._hostmaskCache[id].add(s)
                     except KeyError:
-                        self._hostmaskCache[id] = sets.Set([s])
+                        self._hostmaskCache[id] = set([s])
                     return id
                 elif len(ids) == 0:
                     raise KeyError, s
@@ -656,8 +649,7 @@ class UsersDictionary(utils.IterableMap):
                     log.error('Multiple matches found in user database.  '
                               'Removing the offending hostmasks.')
                     for (id, hostmask) in ids.iteritems():
-                        log.error('Removing %s from user %s.',
-                                  utils.quoted(hostmask), id)
+                        log.error('Removing %q from user %s.', hostmask, id)
                         self.users[id].removeHostmask(hostmask)
                     raise DuplicateHostmask, 'Ids %r matched.' % ids
         else: # Not a hostmask, must be a name.
@@ -715,7 +707,7 @@ class UsersDictionary(utils.IterableMap):
                     del self._hostmaskCache[hostmask]
                 del self._hostmaskCache[id]
 
-    def setUser(self, user):
+    def setUser(self, user, flush=True):
         """Sets a user (given its id) to the IrcUser given it."""
         self.nextId = max(self.nextId, user.id)
         try:
@@ -739,7 +731,8 @@ class UsersDictionary(utils.IterableMap):
                         raise DuplicateHostmask, hostmask
         self.invalidateCache(user.id)
         self.users[user.id] = user
-        self.flush()
+        if flush:
+            self.flush()
 
     def delUser(self, id):
         """Removes a user from the database."""
@@ -779,9 +772,12 @@ class ChannelsDictionary(utils.IterableMap):
                 reader.readFile(filename)
                 self.noFlush = False
                 self.flush()
-            except Exception, e:
+            except EnvironmentError, e:
                 log.error('Invalid channel database, resetting to empty.')
                 log.error('Exact error: %s', utils.exnToString(e))
+            except Exception, e:
+                log.error('Invalid channel database, resetting to empty.')
+                log.exception('Exact error:')
         finally:
             self.noFlush = False
 
@@ -789,7 +785,7 @@ class ChannelsDictionary(utils.IterableMap):
         """Flushes the channel database to its file."""
         if not self.noFlush:
             if self.filename is not None:
-                fd = utils.transactionalFile(self.filename)
+                fd = utils.file.AtomicFile(self.filename)
                 for (channel, c) in self.channels.iteritems():
                     fd.write('channel %s' % channel)
                     fd.write(os.linesep)
@@ -845,7 +841,7 @@ class IgnoresDB(object):
     def open(self, filename):
         self.filename = filename
         fd = file(self.filename)
-        for line in utils.nonCommentNonEmptyLines(fd):
+        for line in utils.file.nonCommentNonEmptyLines(fd):
             try:
                 line = line.rstrip('\r\n')
                 L = line.split()
@@ -856,13 +852,12 @@ class IgnoresDB(object):
                     expiration = 0
                 self.add(hostmask, expiration)
             except Exception, e:
-                log.error('Invalid line in ignores database: %s',
-                          utils.quoted(line))
+                log.error('Invalid line in ignores database: %q', line)
         fd.close()
 
     def flush(self):
         if self.filename is not None:
-            fd = utils.transactionalFile(self.filename)
+            fd = utils.file.AtomicFile(self.filename)
             now = time.time()
             for (hostmask, expiration) in self.hostmasks.items():
                 if now < expiration or not expiration:
@@ -1055,8 +1050,8 @@ def checkCapability(hostmask, capability, users=users, channels=channels):
 def checkCapabilities(hostmask, capabilities, requireAll=False):
     """Checks that a user has capabilities in a list.
 
-    requireAll is the True if *all* capabilities in the list must be had, False
-    if *any* of the capabilities in the list must be had.
+    requireAll is True if *all* capabilities in the list must be had, False if
+    *any* of the capabilities in the list must be had.
     """
     for capability in capabilities:
         if requireAll:
@@ -1102,4 +1097,4 @@ conf.registerGlobalValue(conf.supybot.capabilities, 'default',
     have the capability for whatever command he wishes to run."""))
 
 
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
