@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004 Jeremiah Fincher
+# Copyright (c) 2002-2005 Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -27,16 +27,9 @@
 # POSSIBILITY OF SUCH DAMAGE.
 ###
 
-__revision__ = "$Id: irclib.py,v 1.205 2005/01/10 20:41:23 jemfinch Exp $"
-
-import supybot.fix as fix
-
 import copy
-import sets
 import time
 import random
-import operator
-from itertools import imap, chain, cycle
 
 import supybot.log as log
 import supybot.conf as conf
@@ -45,7 +38,10 @@ import supybot.world as world
 import supybot.ircdb as ircdb
 import supybot.ircmsgs as ircmsgs
 import supybot.ircutils as ircutils
-from supybot.structures import queue, smallqueue, RingBuffer
+
+from utils.str import rsplit
+from utils.iter import imap, chain, cycle
+from utils.structures import queue, smallqueue, RingBuffer
 
 ###
 # The base class for a callback to be registered with an Irc object.  Shows
@@ -76,13 +72,18 @@ class IrcCallback(IrcCommandDispatcher):
     __firewalled__ = {'die': None,
                       'reset': None,
                       '__call__': None,
-                      '__lt__': lambda self: 0,
                       'inFilter': lambda self, irc, msg: msg,
                       'outFilter': lambda self, irc, msg: msg,
-                      'name': lambda self: self.__class__.__name__,}
+                      'name': lambda self: self.__class__.__name__,
+                      'callPrecedence': lambda self, irc: ([], []),
+                      }
+
+    def __init__(self, *args, **kwargs):
+        super(IrcCallback, self).__init__(*args, **kwargs)
 
     def __repr__(self):
-        return '<%s %s>' % (self.__class__.__name__, self.name())
+        return '<%s %s %s>' % \
+               (self.__class__.__name__, self.name(), object.__repr__(self))
 
     def name(self):
         """Returns the name of the callback."""
@@ -104,7 +105,7 @@ class IrcCallback(IrcCommandDispatcher):
         assert self not in after, '%s was in its own after.' % self.name()
         assert self not in before, '%s was in its own before.' % self.name()
         return (before, after)
-                
+
     def inFilter(self, irc, msg):
         """Used for filtering/modifying messages as they're entering.
 
@@ -139,8 +140,8 @@ class IrcCallback(IrcCommandDispatcher):
 # Basic queue for IRC messages.  It doesn't presently (but should at some
 # later point) reorder messages based on priority or penalty calculations.
 ###
-_high = sets.ImmutableSet(['MODE', 'KICK', 'PONG', 'NICK', 'PASS', 'CAPAB'])
-_low = sets.ImmutableSet(['PRIVMSG', 'PING', 'WHO', 'NOTICE'])
+_high = frozenset(['MODE', 'KICK', 'PONG', 'NICK', 'PASS', 'CAPAB'])
+_low = frozenset(['PRIVMSG', 'PING', 'WHO', 'NOTICE', 'JOIN'])
 class IrcMsgQueue(object):
     """Class for a queue of IrcMsgs.  Eventually, it should be smart.
 
@@ -154,7 +155,7 @@ class IrcMsgQueue(object):
     the 'high priority' ones before the normal ones before the 'low priority'
     ones.
     """
-    __slots__ = ('msgs', 'highpriority', 'normal', 'lowpriority')
+    __slots__ = ('msgs', 'highpriority', 'normal', 'lowpriority', 'lastJoin')
     def __init__(self, iterable=()):
         self.reset()
         for msg in iterable:
@@ -162,21 +163,19 @@ class IrcMsgQueue(object):
 
     def reset(self):
         """Clears the queue."""
+        self.lastJoin = 0
         self.highpriority = smallqueue()
         self.normal = smallqueue()
         self.lowpriority = smallqueue()
-        self.msgs = sets.Set()
 
     def enqueue(self, msg):
         """Enqueues a given message."""
-        if msg in self.msgs and \
-           not conf.supybot.protocols.irc.queueDuplicateMessages():
+        if msg in self and \
+           conf.supybot.protocols.irc.queuing.duplicates():
             s = str(msg).strip()
-            log.info('Not adding message %s to queue, already added.',
-                     utils.quoted(s))
+            log.info('Not adding message %q to queue, already added.', s)
             return False
         else:
-            self.msgs.add(msg)
             if msg.command in _high:
                 self.highpriority.enqueue(msg)
             elif msg.command in _low:
@@ -194,19 +193,26 @@ class IrcMsgQueue(object):
             msg = self.normal.dequeue()
         elif self.lowpriority:
             msg = self.lowpriority.dequeue()
-        if msg:
-            try:
-                self.msgs.remove(msg)
-            except KeyError:
-                s = 'Odd, dequeuing a message that\'s not in self.msgs.'
-                log.warning(s)
+            if msg.command == 'JOIN':
+                limit = conf.supybot.protocols.irc.queuing.rateLimit.join()
+                now = time.time()
+                if self.lastJoin + limit <= now:
+                    self.lastJoin = now
+                else:
+                    self.lowpriority.enqueue(msg)
+                    msg = None
         return msg
 
+    def __contains__(self, msg):
+        return msg in self.normal or \
+               msg in self.lowpriority or \
+               msg in self.highpriority
+
     def __nonzero__(self):
         return bool(self.highpriority or self.normal or self.lowpriority)
 
     def __len__(self):
-        return sum(imap(len,[self.highpriority,self.lowpriority,self.normal]))
+        return len(self.highpriority)+len(self.lowpriority)+len(self.normal)
 
     def __repr__(self):
         name = self.__class__.__name__
@@ -239,7 +245,7 @@ class ChannelState(object):
         return nick in self.voices
     def isHalfop(self, nick):
         return nick in self.halfops
-    
+
     def addUser(self, user):
         "Adds a given user to the ChannelState.  Power prefixes are handled."
         nick = user.lstrip('@%+')
@@ -285,24 +291,24 @@ class ChannelState(object):
     def doMode(self, msg):
         def getSet(c):
             if c == 'o':
-                set = self.ops
+                Set = self.ops
             elif c == 'v':
-                set = self.voices
+                Set = self.voices
             elif c == 'h':
-                set = self.halfops
+                Set = self.halfops
             elif c == 'b':
-                set = self.bans
+                Set = self.bans
             else: # We don't care yet, so we'll just return an empty set.
-                set = sets.Set()
-            return set
+                Set = set()
+            return Set
         for (mode, value) in ircutils.separateModes(msg.args[1:]):
             (action, modeChar) = mode
             if modeChar in 'ovhbeq': # We don't handle e or q yet.
-                set = getSet(modeChar)
+                Set = getSet(modeChar)
                 if action == '-':
-                    set.discard(value)
+                    Set.discard(value)
                 elif action == '+':
-                    set.add(value)
+                    Set.add(value)
             else:
                 if action == '+':
                     self.setMode(modeChar, value)
@@ -358,7 +364,7 @@ class IrcState(IrcCommandDispatcher):
     def __reduce__(self):
         return (self.__class__, (self.history, self.supported,
                                  self.nicksToHostmasks, self.channels))
-    
+
     def __eq__(self, other):
         return self.history == other.history and \
                self.channels == other.channels and \
@@ -395,7 +401,6 @@ class IrcState(IrcCommandDispatcher):
     _005converters = utils.InsensitivePreservingDict({
         'modes': int,
         'keylen': int,
-        'maxbans': int,
         'nicklen': int,
         'userlen': int,
         'hostlen': int,
@@ -420,6 +425,34 @@ class IrcState(IrcCommandDispatcher):
             return dict(zip('ovh', s))
     _005converters['prefix'] = _prefixParser
     del _prefixParser
+    def _maxlistParser(s):
+        modes = ''
+        limits = []
+        pairs = s.split(',')
+        for pair in pairs:
+            (mode, limit) = pair.split(':', 1)
+            modes += mode
+            limits += (int(limit),) * len(mode)
+        return dict(zip(modes, limits))
+    _005converters['maxlist'] = _maxlistParser
+    del _maxlistParser
+    def _maxbansParser(s):
+        # IRCd using a MAXLIST style string (IRCNet)
+        if ':' in s:
+            modes = ''
+            limits = []
+            pairs = s.split(',')
+            for pair in pairs:
+                (mode, limit) = pair.split(':', 1)
+                modes += mode
+                limits += (int(limit),) * len(mode)
+            d = dict(zip(modes, limits))
+            assert 'b' in d
+            return d['b']
+        else:
+            return int(s)
+    _005converters['maxbans'] = _maxbansParser
+    del _maxbansParser
     def do005(self, irc, msg):
         for arg in msg.args[1:-1]: # 0 is nick, -1 is "are supported"
             if '=' in arg:
@@ -432,7 +465,7 @@ class IrcState(IrcCommandDispatcher):
                     log.error('Name: %s, Converter: %s', name, converter)
             else:
                 self.supported[arg] = None
-        
+
     def do352(self, irc, msg):
         # WHO reply.
         (nick, user, host) = (msg.args[5], msg.args[2], msg.args[3])
@@ -502,7 +535,11 @@ class IrcState(IrcCommandDispatcher):
         (channel, users) = msg.args[:2]
         chan = self.channels[channel]
         for user in users.split(','):
-            chan.removeUser(user)
+            if ircutils.strEqual(user, irc.nick):
+                del self.channels[channel]
+                return
+            else:
+                chan.removeUser(user)
 
     def doQuit(self, irc, msg):
         for channel in self.channels.itervalues():
@@ -556,9 +593,9 @@ class Irc(IrcCommandDispatcher):
     __firewalled__ = {'die': None,
                       'feedMsg': None,
                       'takeMsg': None,}
-    _nickSetters = sets.Set(['001', '002', '003', '004', '250', '251', '252',
-                             '254', '255', '265', '266', '372', '375', '376',
-                             '333', '353', '332', '366', '005'])
+    _nickSetters = set(['001', '002', '003', '004', '250', '251', '252',
+                        '254', '255', '265', '266', '372', '375', '376',
+                        '333', '353', '332', '366', '005'])
     # We specifically want these callbacks to be common between all Ircs,
     # that's why we don't do the normal None default with a check.
     def __init__(self, network, callbacks=_callbacks):
@@ -593,11 +630,12 @@ class Irc(IrcCommandDispatcher):
     # This *isn't* threadsafe!
     def addCallback(self, callback):
         """Adds a callback to the callbacks list."""
+        assert not self.getCallback(callback.name())
         self.callbacks.append(callback)
         # This is the new list we're building, which will be tsorted.
         cbs = []
         # The vertices are self.callbacks itself.  Now we make the edges.
-        edges = sets.Set()
+        edges = set()
         for cb in self.callbacks:
             (before, after) = cb.callPrecedence(self)
             assert cb not in after, 'cb was in its own after.'
@@ -607,7 +645,7 @@ class Irc(IrcCommandDispatcher):
             for otherCb in after:
                 edges.add((cb, otherCb))
         def getFirsts():
-            firsts = sets.Set(self.callbacks) - sets.Set(cbs)
+            firsts = set(self.callbacks) - set(cbs)
             for (before, after) in edges:
                 firsts.discard(after)
             return firsts
@@ -642,7 +680,7 @@ class Irc(IrcCommandDispatcher):
         name = name.lower()
         def nameMatches(cb):
             return cb.name().lower() == name
-        (bad, good) = partition(nameMatches, self.callbacks)
+        (bad, good) = utils.iter.partition(nameMatches, self.callbacks)
         self.callbacks[:] = good
         return bad
 
@@ -682,7 +720,6 @@ class Irc(IrcCommandDispatcher):
                 s = 'Ping sent at %s not replied to.' % \
                     log.timestamp(self.lastping)
                 log.warning(s)
-                # Let's notify the plugins that we're reconnecting.
                 self.feedMsg(ircmsgs.error(s))
                 self.driver.reconnect()
             elif not self.zombie:
@@ -694,7 +731,7 @@ class Irc(IrcCommandDispatcher):
             for callback in reversed(self.callbacks):
                 msg = callback.outFilter(self, msg)
                 if msg is None:
-                    log.debug('%s.outFilter returned None.' % callback.name())
+                    log.debug('%s.outFilter returned None.', callback.name())
                     return self.takeMsg()
                 world.debugFlush()
             if len(str(msg)) > 512:
@@ -714,12 +751,7 @@ class Irc(IrcCommandDispatcher):
             # On second thought, we need this for testing.
             if world.testing:
                 self.state.addMsg(self, msg)
-            log.debug('Outgoing message: ' + str(msg).rstrip('\r\n'))
-            if msg.command == 'JOIN':
-                channels = msg.args[0].split(',')
-                for channel in channels:
-                    # Let's make this more accurate.
-                    self.startedSync[channel] = time.time()
+            log.debug('Outgoing message: %s', str(msg).rstrip('\r\n'))
             return msg
         elif self.zombie:
             # We kill the driver here so it doesn't continue to try to
@@ -737,7 +769,8 @@ class Irc(IrcCommandDispatcher):
             channel = msg.args[0]
         else:
             channel = None
-        log.debug('Incoming message: ' + str(msg).rstrip('\r\n'))
+        preInFilter = str(msg).rstrip('\r\n')
+        log.debug('Incoming message: %s', preInFilter)
 
         # Yeah, so this is odd.  Some networks (oftc) seem to give us certain
         # messages with our nick instead of our prefix.  We'll fix that here.
@@ -757,7 +790,7 @@ class Irc(IrcCommandDispatcher):
                 log.debug('Updating nick attribute to %s.', self.nick)
             if msg.prefix != self.server:
                 self.server = msg.prefix
-                log.debug('Updating server attribute to %s.')
+                log.debug('Updating server attribute to %s.', self.server)
 
         # Dispatch to specific handlers for commands.
         method = self.dispatchCommand(msg.command)
@@ -776,12 +809,15 @@ class Irc(IrcCommandDispatcher):
             try:
                 m = callback.inFilter(self, msg)
                 if not m:
-                    log.debug('%s.inFilter returned None' % callback.name())
+                    log.debug('%s.inFilter returned None', callback.name())
                     return
                 msg = m
             except:
                 log.exception('Uncaught exception in inFilter:')
             world.debugFlush()
+        postInFilter = str(msg).rstrip('\r\n')
+        if postInFilter != preInFilter:
+            log.debug('Incoming message (post-inFilter): %s', postInFilter)
         for callback in self.callbacks:
             try:
                 if callback is not None:
@@ -855,7 +891,7 @@ class Irc(IrcCommandDispatcher):
             while len(L) <= 3:
                 L.append('`')
             while ircutils.strEqual(ret, nick):
-                L[random.randrange(len(L))] = random.choice('0123456789')
+                L[random.randrange(len(L))] = utils.iter.choice('0123456789')
                 ret = ''.join(L)
             return ret
 
@@ -885,14 +921,16 @@ class Irc(IrcCommandDispatcher):
             self.sendMsg(ircmsgs.mode(self.nick, umodes))
     do377 = do422 = do376
 
-    def do433(self, msg):
-        """Handles 'nickname already in use' messages."""
+    def do43x(self, msg, problem):
         if not self.afterConnect:
             newNick = self._getNextNick()
             assert newNick != self.nick
-            log.info('Got 433: %s is in use.  Trying %s.', self.nick, newNick)
+            log.info('Got 433: %s %s.  Trying %s.',self.nick, problem, newNick)
             self.sendMsg(ircmsgs.nick(newNick))
-    do432 = do433 # 432: Erroneous nickname.
+    def do433(self, msg):
+        self.do43x(msg, 'is in use')
+    def do432(self, msg):
+        self.do43x(msg, 'is not a valid nickname')
 
     def doJoin(self, msg):
         if msg.nick == self.nick:
@@ -903,16 +941,12 @@ class Irc(IrcCommandDispatcher):
 
     def do315(self, msg):
         channel = msg.args[1]
-        popped = False
         if channel in self.startedSync:
             now = time.time()
             started = self.startedSync.pop(channel)
             elapsed = now - started
             log.info('Join to %s on %s synced in %.2f seconds.',
                      channel, self.network, elapsed)
-            popped = True
-        if popped and not self.startedSync:
-            log.info('All channels synced on %s.', self.network)
 
     def doError(self, msg):
         """Handles ERROR messages."""
@@ -950,7 +984,7 @@ class Irc(IrcCommandDispatcher):
 
     def _reallyDie(self):
         """Makes the Irc object die.  Dead."""
-        log.info('Irc object for %s dying.' % self.network)
+        log.info('Irc object for %s dying.', self.network)
         # XXX This hasattr should be removed, I'm just putting it here because
         #     we're so close to a release.  After 0.80.0 we should remove this
         #     and fix whatever AttributeErrors arise in the drivers themselves.
@@ -976,7 +1010,12 @@ class Irc(IrcCommandDispatcher):
         return id(self)
 
     def __eq__(self, other):
-        return id(self) == id(other)
+        # We check isinstance here, so that if some proxy object (like those
+        # defined in callbacks.py) has overridden __eq__, it takes precedence.
+        if isinstance(other, self.__class__):
+            return id(self) == id(other)
+        else:
+            return other == self
 
     def __ne__(self, other):
         return not (self == other)
@@ -988,4 +1027,4 @@ class Irc(IrcCommandDispatcher):
         return '<irclib.Irc object for %s>' % self.network
 
 
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
