@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004, Jeremiah Fincher
+# Copyright (c) 2002-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -34,16 +34,12 @@ construct such messages in an easier way than the constructor for the IrcMsg
 object (which, as you'll read later, is quite...full-featured :))
 """
 
-__revision__ = "$Id: ircmsgs.py,v 1.89 2004/12/30 02:43:21 jamessan Exp $"
-
-import supybot.fix as fix
-
 import re
 import time
-import string
 
 import supybot.conf as conf
 import supybot.utils as utils
+from supybot.utils.iter import all
 import supybot.ircutils as ircutils
 
 ###
@@ -191,9 +187,8 @@ class IrcMsg(object):
     def __repr__(self):
         if self._repr is not None:
             return self._repr
-        self._repr = 'IrcMsg(prefix=%s, command=%s, args=%r)' % \
-                     (utils.quoted(self.prefix), utils.quoted(self.command),
-                      self.args)
+        self._repr = format('IrcMsg(prefix=%q, command=%q, args=%r)',
+                            self.prefix, self.command, self.args)
         return self._repr
 
     def __reduce__(self):
@@ -226,6 +221,19 @@ def isAction(msg):
     else:
         return False
 
+def isSplit(msg):
+    if msg.command == 'QUIT':
+        # It's a quit.
+        quitmsg = msg.args[0]
+        if not quitmsg.startswith('"') and not quitmsg.endswith('"'):
+            # It's not a user-generated quitmsg.
+            servers = quitmsg.split()
+            if len(servers) == 2:
+                # We could check if domains match, or if the hostnames actually
+                # resolve, but we're going to put that off for now.
+                return True
+    return False
+
 _unactionre = re.compile(r'^\x01ACTION\s+(.*)\x01$')
 def unAction(msg):
     """Returns the payload (i.e., non-ACTION text) of an ACTION msg."""
@@ -586,7 +594,8 @@ def join(channel, key=None, prefix='', msg=None):
         return IrcMsg(prefix=prefix, command='JOIN', args=(channel,), msg=msg)
     else:
         if conf.supybot.protocols.irc.strictRfc():
-            assert key.translate(string.ascii, string.ascii[128:]) == key and \
+            assert key.translate(utils.str.chars,
+                                 utils.str.chars[128:]) == key and \
                    '\x00' not in key and \
                    '\r' not in key and \
                    '\n' not in key and \
@@ -613,7 +622,8 @@ def joins(channels, keys=None, prefix='', msg=None):
     else:
         for key in keys:
             if conf.supybot.protocols.irc.strictRfc():
-                assert key.translate(string.ascii,string.ascii[128:])==key and\
+                assert key.translate(utils.str.chars,
+                                     utils.str.chars[128:])==key and \
                        '\x00' not in key and \
                        '\r' not in key and \
                        '\n' not in key and \
@@ -764,4 +774,4 @@ def ison(nick, prefix='', msg=None):
 def error(s, msg=None):
     return IrcMsg(command='ERROR', args=(s,), msg=msg)
 
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
