@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004, Jeremiah Fincher
+# Copyright (c) 2002-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -34,20 +34,14 @@ nick class to handle nicks (so comparisons and hashing and whatnot work in an
 IRC-case-insensitive fashion), and numerous other things.
 """
 
-__revision__ = "$Id: ircutils.py,v 1.119 2004/12/20 19:19:29 jemfinch Exp $"
-
-import supybot.fix as fix
-
 import re
 import time
 import random
 import string
 import textwrap
-from itertools import imap, ilen
 from cStringIO import StringIO as sio
 
 import supybot.utils as utils
-import supybot.structures as structures
 
 def debug(s, *args):
     """Prints a debug string.  Most likely replaced by our logging debug."""
@@ -109,7 +103,7 @@ def toLower(s, casemapping=None):
     elif casemapping == 'ascii': # freenode
         return s.lower()
     else:
-        raise ValueError, 'Invalid casemapping: %s' % utils.quoted(casemapping)
+        raise ValueError, 'Invalid casemapping: %r' % casemapping
 
 def strEqual(nick1, nick2):
     """s1, s2 => bool
@@ -120,10 +114,9 @@ def strEqual(nick1, nick2):
 
 nickEqual = strEqual
 
-_nickchars = r'_[]\`^{}|-'
-nickRe = re.compile(r'^[A-Za-z%s][0-9A-Za-z%s]*$'
+_nickchars = r'[]\`_^{|}'
+nickRe = re.compile(r'^[A-Za-z%s][-0-9A-Za-z%s]*$'
                     % (re.escape(_nickchars), re.escape(_nickchars)))
-
 def isNick(s, strictRfc=True, nicklen=None):
     """s => bool
     Returns True if s is a valid IRC nick."""
@@ -197,11 +190,11 @@ def banmask(hostmask):
     """
     assert isUserHostmask(hostmask)
     host = hostFromHostmask(hostmask)
-    if utils.isIP(host):
+    if utils.net.isIP(host):
         L = host.split('.')
         L[-1] = '*'
         return '*!*@' + '.'.join(L)
-    elif utils.isIPV6(host):
+    elif utils.net.isIPV6(host):
         L = host.split(':')
         L[-1] = '*'
         return '*!*@' + ':'.join(L)
@@ -450,7 +443,7 @@ def safeArgument(s):
     if isinstance(s, unicode):
         s = s.encode('utf-8')
     elif not isinstance(s, basestring):
-        debug('Got a non-string in safeArgument: %s', utils.quoted(s))
+        debug('Got a non-string in safeArgument: %r', s)
         s = str(s)
     if isValidArgument(s):
         return s
@@ -466,7 +459,7 @@ def replyTo(msg):
 
 def dccIP(ip):
     """Returns in IP in the proper for DCC."""
-    assert utils.isIP(ip), \
+    assert utils.net.isIP(ip), \
            'argument must be a string ip in xxx.yyy.zzz.www format.'
     i = 0
     x = 256**3
@@ -483,7 +476,7 @@ def unDccIP(i):
         L.append(i % 256)
         i /= 256
     L.reverse()
-    return '.'.join(imap(str, L))
+    return '.'.join(utils.iter.imap(str, L))
 
 class IrcString(str):
     """This class does case-insensitive comparison and hashing of nicks."""
@@ -556,7 +549,7 @@ class FloodQueue(object):
                 # calls structures.TimeoutQueue.__repr__, which calls
                 # getTimeout.__repr__, which calls our __repr__, which calls...
                 getTimeout = lambda : self.getTimeout()
-                q = structures.TimeoutQueue(getTimeout)
+                q = utils.structures.TimeoutQueue(getTimeout)
                 self.queues[key] = q
                 return q
             else:
@@ -630,7 +623,7 @@ def standardSubstitute(irc, msg, text, env=None):
             if len(L) > 1:
                 n = msg.nick
                 while n == msg.nick:
-                    n = random.choice(L)
+                    n = utils.iter.choice(L)
                 return n
             else:
                 return msg.nick
@@ -662,10 +655,10 @@ def standardSubstitute(irc, msg, text, env=None):
         })
     if env is not None:
         vars.update(env)
-    return utils.perlVariableSubstitute(vars, text)
+    return utils.str.perlVariableSubstitute(vars, text)
 
 
 if __name__ == '__main__':
     import sys, doctest
     doctest.testmod(sys.modules['__main__'])
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
