@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004, Jeremiah Fincher
+# Copyright (c) 2002-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -27,10 +27,6 @@
 # POSSIBILITY OF SUCH DAMAGE.
 ###
 
-__revision__ = "$Id: log.py,v 1.66 2004/12/22 09:05:38 jemfinch Exp $"
-
-import supybot.fix as fix
-
 import os
 import sys
 import time
@@ -56,9 +52,6 @@ deadlyExceptions = [KeyboardInterrupt, SystemExit]
 ###
 testing = False
 
-VERBOSE = 1
-logging.addLevelName(VERBOSE, 'VERBOSE')
-
 class Formatter(logging.Formatter):
     _fmtConf = staticmethod(lambda : conf.supybot.log.format())
     def formatTime(self, record, datefmt=None):
@@ -87,15 +80,13 @@ class Logger(logging.Logger):
         eStrId = '%s:%s' % (E, path)
         eId = hex(hash(eStrId) & 0xFFFFF)
         logging.Logger.exception(self, *args)
-        if hasattr(e, '__revision__') and e.__revision__:
-            self.error('Exception __revision__: %s',
-                       utils.quoted(e.__revision__))
         self.error('Exception id: %s', eId)
         # The traceback should be sufficient if we want it.
         # self.error('Exception string: %s', eStrId)
 
-    def verbose(self, *args, **kwargs):
-        self.log(VERBOSE, *args, **kwargs)
+    def _log(self, level, msg, args, exc_info=None):
+        msg = format(msg, *args)
+        logging.Logger._log(self, level, msg, (), exc_info=exc_info)
 
 
 class StdoutStreamHandler(logging.StreamHandler):
@@ -211,7 +202,7 @@ class ValidLogLevel(registry.String):
         return level
 
 class LogLevel(ValidLogLevel):
-    """Invalid log level.  Value must be either VERBOSE, DEBUG, INFO, WARNING,
+    """Invalid log level.  Value must be either DEBUG, INFO, WARNING,
     ERROR, or CRITICAL."""
     def setValue(self, v):
         ValidLogLevel.setValue(self, v)
@@ -229,13 +220,8 @@ conf.registerGlobalValue(conf.supybot.log, 'format',
     its logging module."""))
 conf.registerGlobalValue(conf.supybot.log, 'level',
     LogLevel(logging.INFO, """Determines what the minimum priority level logged
-    will be.  Valid values are VERBOSE, DEBUG, INFO, WARNING, ERROR,
+    will be.  Valid values are DEBUG, INFO, WARNING, ERROR,
     and CRITICAL, in order of increasing priority."""))
-conf.registerGlobalValue(conf.supybot.log, 'statistics',
-    ValidLogLevel(-1, """Determines what level statistics reporting
-    is to be logged at.  Mostly, this just includes, for instance, the time it
-    took to parse a message, process a command, etc.  You probably don't care
-    about this."""))
 conf.registerGlobalValue(conf.supybot.log, 'timestampFormat',
     registry.String('%Y-%m-%dT%H:%M:%S', """Determines the format string for
     timestamps in logfiles.  Refer to the Python documentation for the time
@@ -277,7 +263,6 @@ conf.registerGlobalValue(conf.supybot.log.plugins, 'format',
 
 # These just make things easier.
 debug = _logger.debug
-verbose = _logger.verbose
 info = _logger.info
 warning = _logger.warning
 error = _logger.error
@@ -288,10 +273,6 @@ exception = _logger.exception
 registry.error = error
 registry.exception = exception
 
-def stat(*args):
-    level = conf.supybot.log.statistics()
-    _logger.log(level, *args)
-
 setLevel = _logger.setLevel
 
 atexit.register(logging.shutdown)
@@ -310,18 +291,7 @@ def getPluginLogger(name):
         handler.setFormatter(pluginFormatter)
         log.addHandler(handler)
     if name in sys.modules:
-        # Let's log the version, this might be useful.
-        module = sys.modules[name]
-        try:
-            if hasattr(module, '__revision__'):
-                version = module.__revision__.split()[2]
-                log.info('Starting log for %s (revision %s)', name, version)
-            else:
-                debug('Module %s has no __revision__ string.', name)
-                log.info('Starting log for %s.', name)
-        except IndexError:
-            log.debug('Improper __revision__ string in %s.', name)
-            log.info('Starting log for %s.', name)
+        log.info('Starting log for %s.', name)
     return log
 
 def timestamp(when=None):
@@ -355,22 +325,16 @@ def firewall(f, errorHandler=None):
                 except Exception, e:
                     logException(self, 'Uncaught exception in errorHandler')
 
-    m = utils.changeFunctionName(m, f.func_name, f.__doc__)
+    m = utils.python.changeFunctionName(m, f.func_name, f.__doc__)
     return m
 
 class MetaFirewall(type):
     def __new__(cls, name, bases, dict):
-        firewalled = {}
-        for base in bases:
-            if hasattr(base, '__firewalled__'):
-                firewalled.update(base.__firewalled__)
         if '__firewalled__' in dict:
-            firewalled.update(dict['__firewalled__'])
-        for attr in firewalled:
-            if attr in dict:
+            for attr in dict['__firewalled__']:
                 try:
                     errorHandler = firewalled[attr]
-                except:
+                except: # This is raw here so people can still use tuples. 
                     errorHandler = None
                 dict[attr] = firewall(dict[attr], errorHandler)
         return super(MetaFirewall, cls).__new__(cls, name, bases, dict)
@@ -387,15 +351,15 @@ if not os.path.exists(pluginLogDir):
     os.mkdir(pluginLogDir, 0755)
 
 try:
-    miscLogFilename = os.path.join(_logDir, 'misc.log')
-    _handler = BetterFileHandler(miscLogFilename)
+    messagesLogFilename = os.path.join(_logDir, 'messages.log')
+    _handler = BetterFileHandler(messagesLogFilename)
 except EnvironmentError, e:
     raise SystemExit, \
-          'Error opening miscellaneous logfile (%s).  ' \
+          'Error opening messages logfile (%s).  ' \
           'Generally, this is because you are running Supybot in a directory ' \
           'you don\'t have permissions to add files in, or you\'re running ' \
           'Supybot as a different user than you normal do.  The original ' \
-          'error was: %s' % (miscLogFilename, utils.exnToString(e))
+          'error was: %s' % (messagesLogFilename, utils.gen.exnToString(e))
 _handler.setFormatter(formatter)
 _handler.setLevel(-1)
 class PluginLogFilter(logging.Filter):
@@ -417,5 +381,5 @@ if not conf.daemonized:
     _logger.addHandler(_stdoutHandler)
 
 
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
 
