@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2004, Jeremiah Fincher
+# Copyright (c) 2004-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -27,21 +27,17 @@
 # POSSIBILITY OF SUCH DAMAGE.
 ###
 
-__revision__ = "$Id: registry.py,v 1.124 2005/01/14 02:20:53 jamessan Exp $"
-
 import re
 import os
-import sets
 import time
 import string
 import textwrap
 
-import supybot.fix as fix
 import supybot.utils as utils
 
 def error(s):
-    """Replace me with something better from another module!"""
-    print '***', s
+   """Replace me with something better from another module!"""
+   print '***', s
 
 def exception(s):
     """Ditto!"""
@@ -70,7 +66,7 @@ def open(filename, clear=False):
     if clear:
         _cache.clear()
     _fd = file(filename)
-    fd = utils.nonCommentNonEmptyLines(_fd)
+    fd = utils.file.nonCommentNonEmptyLines(_fd)
     acc = ''
     for line in fd:
         line = line.rstrip('\r\n')
@@ -96,7 +92,7 @@ def open(filename, clear=False):
 
 def close(registry, filename, private=True):
     first = True
-    fd = utils.transactionalFile(filename)
+    fd = utils.file.AtomicFile(filename)
     for (name, value) in registry.getValues(getChildren=True):
         help = value.help()
         if help:
@@ -164,7 +160,7 @@ class Group(object):
     """A group; it doesn't hold a value unless handled by a subclass."""
     def __init__(self, help='', supplyDefault=False,
                  orderAlphabetically=False, private=False):
-        self._help = help
+        self._help = utils.str.normalizeWhitespace(help)
         self._name = 'unset'
         self._added = []
         self._children = utils.InsensitivePreservingDict()
@@ -301,7 +297,7 @@ class Value(Group):
         self.__parent.__init__(help, **kwargs)
         self._default = default
         self._showDefault = showDefault
-        self._help = utils.normalizeWhitespace(help.strip())
+        self._help = utils.str.normalizeWhitespace(help.strip())
         if setDefault:
             self.setValue(default)
 
@@ -310,8 +306,9 @@ class Value(Group):
             s = self.__doc__
         else:
             s = """%s has no docstring.  If you're getting this message,
-            report it, because we forgot to put a proper help string here."""
-        e = InvalidRegistryValue(utils.normalizeWhitespace(s % self._name))
+            report it, because we forgot to put a proper help string here."""%\
+            self._name
+        e = InvalidRegistryValue(utils.str.normalizeWhitespace(s))
         e.value = self
         raise e
 
@@ -358,7 +355,7 @@ class Boolean(Value):
     """Value must be either True or False (or On or Off)."""
     def set(self, s):
         try:
-            v = utils.toBool(s)
+            v = utils.str.toBool(s)
         except ValueError:
             if s.strip().lower() == 'toggle':
                 v = not self.value
@@ -442,7 +439,7 @@ class String(Value):
 
     _printable = string.printable[:-4]
     def _needsQuoting(self, s):
-        return s.translate(string.ascii, self._printable) and s.strip() != s
+        return s.translate(utils.str.chars, self._printable) and s.strip() != s
 
     def __str__(self):
         s = self.value
@@ -457,13 +454,12 @@ class OnlySomeStrings(String):
                                   'This is a bug.'
         self.__parent = super(OnlySomeStrings, self)
         self.__parent.__init__(*args, **kwargs)
-        self.__doc__ = 'Valid values include %s.' % \
-                        utils.commaAndify(map(repr, self.validStrings))
+        self.__doc__ = format('Valid values include %L.',
+                              map(repr, self.validStrings))
 
     def help(self):
         strings = [s for s in self.validStrings if s]
-        return '%s  Valid strings: %s.' % \
-                (self._help, utils.commaAndify(strings))
+        return format('%s  Valid strings: %L.', self._help, strings)
 
     def normalize(self, s):
         lowered = s.lower()
@@ -489,7 +485,7 @@ class NormalizedString(String):
         self._showDefault = False
 
     def normalize(self, s):
-        return utils.normalizeWhitespace(s.strip())
+        return utils.str.normalizeWhitespace(s.strip())
 
     def set(self, s):
         s = self.normalize(s)
@@ -542,7 +538,7 @@ class Regexp(Value):
     def set(self, s):
         try:
             if s:
-                self.setValue(utils.perlReToPythonRe(s), sr=s)
+                self.setValue(utils.str.perlReToPythonRe(s), sr=s)
             else:
                 self.setValue(None)
         except ValueError, e:
@@ -610,7 +606,7 @@ class SpaceSeparatedListOfStrings(SpaceSeparatedListOf):
     Value = String
 
 class SpaceSeparatedSetOfStrings(SpaceSeparatedListOfStrings):
-    List = sets.Set
+    List = set
 
 class CommaSeparatedListOfStrings(SeparatedListOf):
     Value = String
@@ -619,5 +615,5 @@ class CommaSeparatedListOfStrings(SeparatedListOf):
     joiner = ', '.join
 
 
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
 
