@@ -1,5 +1,5 @@
 ###
-# Copyright (c) 2002-2004, Jeremiah Fincher
+# Copyright (c) 2002-2005, Jeremiah Fincher
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -31,10 +31,6 @@
 Module for general worldly stuff, like global variables and whatnot.
 """
 
-__revision__ = "$Id: world.py,v 1.67 2005/01/12 21:07:42 jemfinch Exp $"
-
-import supybot.fix as fix
-
 import gc
 import os
 import sys
@@ -54,8 +50,6 @@ startedAt = time.time() # Just in case it doesn't get set later.
 starting = False
 
 mainThread = threading.currentThread()
-# ??? Should we do this?  What do we gain?
-# assert 'MainThread' in repr(mainThread)
 
 def isMainThread():
     return mainThread is threading.currentThread()
@@ -67,6 +61,7 @@ class SupyThread(threading.Thread):
         global threadsSpawned
         threadsSpawned += 1
         super(SupyThread, self).__init__(*args, **kwargs)
+        log.debug('Spawning thread %q.', self.getName())
         
 commandsProcessed = 0
 
@@ -119,6 +114,12 @@ def upkeep():
         # log.warning it (things shouldn't be printed, and we're more likely
         # to get bug reports if we make it a warning).
         assert not type(sys.stdout) == file, 'Not a StringIO object!'
+        if not hasattr(sys.stdout, 'getvalue'):
+            # Stupid twisted sometimes replaces our stdout with theirs, because
+            # "The Twisted Way Is The Right Way" (ha!).  So we're stuck simply
+            # returning.
+            log.warning('Expected cStringIO as stdout, got %r.', sys.stdout)
+            return
         s = sys.stdout.getvalue()
         if s:
             log.warning('Printed to stdout after daemonization: %s', s)
@@ -139,8 +140,8 @@ def upkeep():
         #    registry.open(registryFilename)
     if not dying:
         log.debug('Regexp cache size: %s', len(sre._cache))
-        log.debug('Pattern cache size: %s'%len(ircutils._patternCache))
-        log.debug('HostmaskPatternEqual cache size: %s' %
+        log.debug('Pattern cache size: %s', len(ircutils._patternCache))
+        log.debug('HostmaskPatternEqual cache size: %s',
                   len(ircutils._hostmaskPatternEqualCache))
         #timestamp = log.timestamp()
         if doFlush:
@@ -199,4 +200,4 @@ profiling = False
 documenting = False
 
 
-# vim:set shiftwidth=4 tabstop=8 expandtab textwidth=78:
+# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
